"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformEnums = exports.transformModelWithFields = exports.transformBareModel = exports.transformMappings = exports.transformSchema = void 0;
const tslib_1 = require("tslib");
const helpers_1 = require("./helpers");
const helpers_2 = require("../helpers");
const pluralize_1 = tslib_1.__importDefault(require("pluralize"));
const config_1 = require("../config");
exports.transformSchema = (datamodel, dmmfDocument) => {
    return {
        inputTypes: datamodel.inputTypes.map(transformInputType(dmmfDocument)),
        outputTypes: datamodel.outputTypes.map(transformOutputType(dmmfDocument)),
        rootMutationType: datamodel.rootMutationType,
        rootQueryType: datamodel.rootQueryType,
    };
};
exports.transformMappings = (mapping, dmmfDocument, options) => {
    return mapping.map(transformMapping(dmmfDocument, options));
};
exports.transformBareModel = (model) => {
    const attributeArgs = helpers_1.parseDocumentationAttributes(model.documentation, 'type', 'model');
    const typeName = attributeArgs === null || attributeArgs === void 0 ? void 0 : attributeArgs.slice(1, -1);
    return {
        ...model,
        typeName: typeName !== null && typeName !== void 0 ? typeName : helpers_2.pascalCase(model.name),
        fields: [],
        docs: helpers_2.cleanDocsString(model.documentation),
    };
};
exports.transformModelWithFields = (dmmfDocument) => {
    return (model) => {
        return {
            ...exports.transformBareModel(model),
            fields: model.fields.map(transformField(dmmfDocument)),
        };
    };
};
const transformField = (dmmfDocument) => {
    return (field) => {
        const attributeArgs = helpers_1.parseDocumentationAttributes(field.documentation, 'field', 'field');
        const typeFieldAlias = attributeArgs === null || attributeArgs === void 0 ? void 0 : attributeArgs.slice(1, -1);
        const fieldTSType = helpers_2.getFieldTSType(field, dmmfDocument, false);
        const typeGraphQLType = helpers_2.getGraphQLType(field, dmmfDocument);
        return {
            ...field,
            typeFieldAlias,
            fieldTSType,
            typeGraphQLType,
            docs: helpers_2.cleanDocsString(field.documentation),
        };
    };
};
const transformInputType = (dmmfDocument) => {
    return (inputType) => {
        const modelName = helpers_2.getModelNameFromInputType(inputType.name);
        const modelType = modelName
            ? dmmfDocument.datamodel.models.find((it) => it.name === modelName)
            : undefined;
        return {
            ...inputType,
            typeName: helpers_2.getInputTypeName(inputType.name, dmmfDocument),
            fields: inputType.fields.map((field) => {
                var _a;
                const modelField = modelType === null || modelType === void 0 ? void 0 : modelType.fields.find((it) => it.name === field.name);
                const typeName = (_a = modelField === null || modelField === void 0 ? void 0 : modelField.typeFieldAlias) !== null && _a !== void 0 ? _a : field.name;
                const selectedInputType = selectInputTypeFromTypes(dmmfDocument)(field.inputType);
                const typeGraphQLType = helpers_2.getGraphQLType(selectedInputType, dmmfDocument);
                const fieldTSType = helpers_2.getFieldTSType(selectedInputType, dmmfDocument, true);
                return {
                    ...field,
                    selectedInputType,
                    typeName,
                    typeGraphQLType,
                    fieldTSType,
                    hasMappedName: field.name !== typeName,
                };
            }),
        };
    };
};
const transformOutputType = (dmmfDocument) => {
    return (outputType) => {
        // TODO: make it more future-proof
        const modelName = outputType.name.replace('Aggregate', '');
        const typeName = getMappedOutputTypeName(dmmfDocument, outputType.name);
        return {
            ...outputType,
            modelName,
            typeName,
            fields: outputType.fields.map((field) => {
                const outputType = {
                    ...field.outputType,
                    type: getMappedOutputTypeName(dmmfDocument, field.outputType.type),
                };
                const fieldTSType = helpers_2.getFieldTSType(outputType, dmmfDocument, false);
                const typeGraphQLType = helpers_2.getGraphQLType(outputType, dmmfDocument);
                const args = field.args.map((arg) => {
                    const selectedInputType = selectInputTypeFromTypes(dmmfDocument)(arg.inputType);
                    const typeGraphQLType = helpers_2.getGraphQLType(selectedInputType, dmmfDocument);
                    const fieldTSType = helpers_2.getFieldTSType(selectedInputType, dmmfDocument, true);
                    return {
                        ...arg,
                        selectedInputType,
                        fieldTSType,
                        typeGraphQLType,
                        hasMappedName: arg.name !== typeName,
                        // TODO: add proper mapping in the future if needed
                        typeName: arg.name,
                    };
                });
                const argsTypeName = args.length > 0
                    ? `${typeName}${helpers_2.pascalCase(field.name)}Args`
                    : undefined;
                return {
                    ...field,
                    outputType,
                    fieldTSType,
                    typeGraphQLType,
                    args,
                    argsTypeName,
                };
            }),
        };
    };
};
const getMappedOutputTypeName = (dmmfDocument, outputTypeName) => {
    if (outputTypeName.startsWith('Aggregate')) {
        return `Aggregate${dmmfDocument.getModelTypeName(outputTypeName.replace('Aggregate', ''))}`;
    }
    const dedicatedTypeSuffix = [
        'MinAggregateOutputType',
        'MaxAggregateOutputType',
        'AvgAggregateOutputType',
        'SumAggregateOutputType',
    ].find((type) => outputTypeName.includes(type));
    if (dedicatedTypeSuffix) {
        const modelName = outputTypeName.replace(dedicatedTypeSuffix, '');
        return `${dmmfDocument.getModelTypeName(modelName)}${dedicatedTypeSuffix}`;
    }
    return outputTypeName;
};
const transformMapping = (dmmfDocument, options) => {
    return (mapping) => {
        const { model, plural, ...availableActions } = mapping;
        const actions = Object.entries(availableActions).map(([modelAction, fieldName]) => {
            var _a;
            const kind = modelAction;
            const modelName = (_a = dmmfDocument.getModelTypeName(model)) !== null && _a !== void 0 ? _a : model;
            const actionOutputType = dmmfDocument.schema.outputTypes.find((type) => type.fields.some((field) => field.name === fieldName));
            if (!actionOutputType) {
                throw new Error(`Cannot find type with field ${fieldName} in root types definitions!`);
            }
            const method = actionOutputType.fields.find((field) => field.name === fieldName);
            const argsTypeName = method.args.length > 0
                ? `${helpers_2.pascalCase(`${kind}${dmmfDocument.getModelTypeName(mapping.model)}`)}Args`
                : undefined;
            const outputTypeName = method.outputType.type;
            return {
                name: getMappedActionName(kind, modelName, options),
                fieldName,
                kind: kind,
                operation: getOperationKindName(kind),
                method,
                argsTypeName,
                outputTypeName,
            };
        });
        return {
            model,
            plural,
            actions,
            collectionName: helpers_2.camelCase(mapping.model),
        };
    };
};
const selectInputTypeFromTypes = (dmmfDocument) => {
    return (inputTypes) => {
        const selectedInputType = inputTypes.find((it) => it.kind === 'object') ||
            inputTypes.find((it) => it.kind === 'enum') ||
            inputTypes[0];
        let inputType = selectedInputType.type;
        if (selectedInputType.kind === 'enum') {
            const enumDef = dmmfDocument.enums.find((it) => it.name === inputType);
            inputType = enumDef.typeName;
        }
        else if (selectedInputType.kind === 'object') {
            inputType = helpers_2.getInputTypeName(inputType, dmmfDocument);
        }
        return {
            ...selectedInputType,
            argType: selectedInputType.type,
            type: inputType,
        };
    };
};
const getMappedActionName = (actionName, typeName, options) => {
    const defaultMappedActionName = `${actionName}${typeName}`;
    if (options.useOriginalMapping) {
        return defaultMappedActionName;
    }
    const hasNoPlural = typeName === pluralize_1.default(typeName);
    if (hasNoPlural) {
        return defaultMappedActionName;
    }
    switch (actionName) {
        case 'findOne': {
            return helpers_2.camelCase(typeName);
        }
        case 'findMany': {
            return pluralize_1.default(helpers_2.camelCase(typeName));
        }
        default: {
            return defaultMappedActionName;
        }
    }
};
const getOperationKindName = (actionName) => {
    if (config_1.supportedQueryActions.includes(actionName))
        return 'Query';
    if (config_1.supportedMutationActions.includes(actionName))
        return 'Mutation';
};
exports.transformEnums = (dmmfDocument) => {
    return (enumDef) => {
        const modelName = enumDef.name.includes('DistinctFieldEnum')
            ? enumDef.name.replace('DistinctFieldEnum', '')
            : undefined;
        const typeName = modelName
            ? `${dmmfDocument.getModelTypeName(modelName)}DistinctFieldEnum`
            : enumDef.name;
        return {
            ...enumDef,
            docs: helpers_2.cleanDocsString(enumDef.documentation),
            typeName,
            valuesMap: enumDef.values.map((value) => ({
                value,
                name: (modelName && dmmfDocument.getModelFieldAlias(modelName, value)) ||
                    value,
            })),
        };
    };
};
//# sourceMappingURL=transform.js.map