"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ts_morph_1 = require("ts-morph");
const path_1 = tslib_1.__importDefault(require("path"));
const helpers_1 = require("./helpers");
const enum_1 = tslib_1.__importDefault(require("./enum"));
const model_type_class_1 = tslib_1.__importDefault(require("./model-type-class"));
const relations_1 = tslib_1.__importDefault(require("./resolvers/relations"));
const type_class_1 = require("./type-class");
const full_crud_1 = tslib_1.__importDefault(require("./resolvers/full-crud"));
const config_1 = require("./config");
const imports_1 = require("./imports");
const saveSourceFile_1 = tslib_1.__importDefault(require("../utils/saveSourceFile"));
const dmmf_document_1 = require("./dmmf/dmmf-document");
const args_class_1 = tslib_1.__importDefault(require("./args-class"));
const generateCode = async (dmmf, options, log = helpers_1.noop) => {
    const baseDirPath = options.outputDirPath;
    const project = new ts_morph_1.Project();
    const resolversDirPath = path_1.default.resolve(baseDirPath, config_1.resolversFolderName);
    const modelNames = dmmf.datamodel.models.map((model) => model.name);
    log('Transforming dmmfDocument...');
    const dmmfDocument = new dmmf_document_1.DmmfDocument(dmmf, options);
    log('Generating enums...');
    const datamodelEnumNames = dmmfDocument.datamodel.enums.map((enumDef) => enumDef.typeName);
    await Promise.all(dmmfDocument.datamodel.enums.map((enumDef) => enum_1.default(project, baseDirPath, enumDef)));
    await Promise.all(dmmfDocument.schema.enums
        // skip enums from datamodel
        .filter((enumDef) => !datamodelEnumNames.includes(enumDef.typeName))
        .map((enumDef) => enum_1.default(project, baseDirPath, enumDef)));
    const emittedEnumNames = [
        ...new Set([
            ...dmmfDocument.schema.enums.map((it) => it.typeName),
            ...dmmfDocument.datamodel.enums.map((it) => it.typeName),
        ]),
    ];
    const enumsBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.enumsFolderName, 'index.ts'), undefined, { overwrite: true });
    imports_1.generateEnumsBarrelFile(enumsBarrelExportSourceFile, emittedEnumNames);
    await saveSourceFile_1.default(enumsBarrelExportSourceFile);
    log('Generating models...');
    await Promise.all(dmmfDocument.datamodel.models.map((model) => model_type_class_1.default(project, baseDirPath, model, dmmfDocument, options)));
    const modelsBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.modelsFolderName, 'index.ts'), undefined, { overwrite: true });
    imports_1.generateModelsBarrelFile(modelsBarrelExportSourceFile, dmmfDocument.datamodel.models.map((it) => it.typeName));
    await saveSourceFile_1.default(modelsBarrelExportSourceFile);
    log('Generating output types...');
    const rootTypes = dmmfDocument.schema.outputTypes.filter((type) => ['Query', 'Mutation'].includes(type.name));
    const outputTypesToGenerate = dmmfDocument.schema.outputTypes.filter(
    // skip generating models and root resolvers
    (type) => !modelNames.includes(type.name) && !rootTypes.includes(type));
    const outputTypesFieldsArgsToGenerate = outputTypesToGenerate
        .map((it) => it.fields)
        .reduce((a, b) => a.concat(b), [])
        .filter((it) => it.argsTypeName);
    await Promise.all(outputTypesToGenerate.map((type) => type_class_1.generateOutputTypeClassFromType(project, resolversDirPath, type, dmmfDocument, options)));
    const outputsBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.outputsFolderName, 'index.ts'), undefined, { overwrite: true });
    imports_1.generateOutputsBarrelFile(outputsBarrelExportSourceFile, outputTypesToGenerate.map((it) => it.typeName), outputTypesFieldsArgsToGenerate.length > 0);
    await saveSourceFile_1.default(outputsBarrelExportSourceFile);
    if (outputTypesFieldsArgsToGenerate.length > 0) {
        log('Generating output types args...');
        await Promise.all(outputTypesFieldsArgsToGenerate.map(async (field) => {
            await args_class_1.default(project, path_1.default.resolve(resolversDirPath, config_1.outputsFolderName), field.args, field.argsTypeName, dmmfDocument, options, 2);
        }));
        const outputsArgsBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.outputsFolderName, config_1.argsFolderName, 'index.ts'), undefined, { overwrite: true });
        imports_1.generateArgsBarrelFile(outputsArgsBarrelExportSourceFile, outputTypesFieldsArgsToGenerate.map((it) => it.argsTypeName));
        await saveSourceFile_1.default(outputsArgsBarrelExportSourceFile);
    }
    log('Generating input types...');
    await Promise.all(dmmfDocument.schema.inputTypes.map((type) => type_class_1.generateInputTypeClassFromType(project, resolversDirPath, type, dmmfDocument, options)));
    const inputsBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.inputsFolderName, 'index.ts'), undefined, { overwrite: true });
    imports_1.generateInputsBarrelFile(inputsBarrelExportSourceFile, dmmfDocument.schema.inputTypes.map((it) => it.typeName));
    await saveSourceFile_1.default(inputsBarrelExportSourceFile);
    log('Generating relation resolvers...');
    const relationResolversData = await Promise.all(dmmfDocument.datamodel.models
        .filter((model) => model.fields.some((field) => field.relationName))
        .map((model) => {
        const outputType = dmmfDocument.schema.outputTypes.find((type) => type.name === model.name);
        const mapping = dmmfDocument.mappings.find((it) => it.model === model.name);
        return relations_1.default(project, baseDirPath, model, mapping, outputType, dmmfDocument, options);
    }));
    if (relationResolversData.length > 0) {
        const relationResolversBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.relationsResolversFolderName, 'index.ts'), undefined, { overwrite: true });
        imports_1.generateResolversBarrelFile('relations', relationResolversBarrelExportSourceFile, relationResolversData);
        await saveSourceFile_1.default(relationResolversBarrelExportSourceFile);
    }
    log('Generating crud resolvers...');
    const crudResolversData = await Promise.all(dmmfDocument.mappings.map((mapping) => {
        const model = dmmfDocument.datamodel.models.find((model) => model.name === mapping.model);
        return full_crud_1.default(project, baseDirPath, mapping, model, modelNames, options, dmmfDocument);
    }));
    const crudResolversBarrelExportSourceFile = project.createSourceFile(path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.crudResolversFolderName, 'index.ts'), undefined, { overwrite: true });
    imports_1.generateResolversBarrelFile('crud', crudResolversBarrelExportSourceFile, crudResolversData);
    await saveSourceFile_1.default(crudResolversBarrelExportSourceFile);
    log('Generating index file');
    const indexSourceFile = project.createSourceFile(baseDirPath + '/index.ts', undefined, { overwrite: true });
    imports_1.generateIndexFile(indexSourceFile, relationResolversData.length > 0);
    await saveSourceFile_1.default(indexSourceFile);
};
exports.default = generateCode;
//# sourceMappingURL=generate-code.js.map