"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toUnixPath = exports.cleanDocsString = exports.getInputTypeName = exports.getModelNameFromInputType = exports.pascalCase = exports.camelCase = exports.mapScalarToType = exports.getGraphQLType = exports.mapScalarToTSType = exports.getFieldTSType = exports.noop = void 0;
const helpers_1 = require("./dmmf/helpers");
exports.noop = () => { };
exports.getFieldTSType = (typeInfo, dmmfDocument, isInputType, modelName, typeName) => {
    let TSType;
    if (typeInfo.kind === 'scalar') {
        TSType = exports.mapScalarToTSType(typeInfo.type, isInputType);
    }
    else if (typeInfo.kind === 'object') {
        if (dmmfDocument.isModelName(typeInfo.type)) {
            TSType = dmmfDocument.getModelTypeName(typeInfo.type);
        }
        else {
            TSType =
                !typeName || !modelName
                    ? exports.getInputTypeName(typeInfo.type, dmmfDocument)
                    : typeInfo.type.replace(modelName, typeName);
        }
    }
    else if (typeInfo.kind === 'enum') {
        TSType = `keyof typeof ${typeInfo.type}`;
    }
    else {
        throw new Error(`Unsupported field type kind: ${typeInfo.kind}`);
    }
    if (typeInfo.isList) {
        if (TSType.includes(' ')) {
            TSType = `Array<${TSType}>`;
        }
        else {
            TSType += '[]';
        }
    }
    if (!typeInfo.isRequired) {
        // FIXME: use properly null for output and undefined for input
        // TSType += " | null | undefined";
        TSType += ' | undefined';
    }
    return TSType;
};
exports.mapScalarToTSType = (scalar, isInputType) => {
    switch (scalar) {
        case 'ID':
        case 'UUID': {
            return 'string';
        }
        case 'String': {
            return 'string';
        }
        case 'Boolean': {
            return 'boolean';
        }
        case 'DateTime': {
            return 'Date';
        }
        case 'Int':
        case 'Float': {
            return 'number';
        }
        case 'Json':
            return isInputType ? 'InputJsonValue' : 'JsonValue';
        default:
            throw new Error(`Unrecognized scalar type: ${scalar}`);
    }
};
exports.getGraphQLType = (typeInfo, dmmfDocument, modelName, typeName) => {
    let GraphQLType;
    if (typeInfo.kind === 'scalar') {
        GraphQLType = exports.mapScalarToType(typeInfo.type);
    }
    else if (typeInfo.kind === 'object') {
        if (dmmfDocument.isModelName(typeInfo.type)) {
            GraphQLType = dmmfDocument.getModelTypeName(typeInfo.type);
        }
        else {
            GraphQLType =
                !typeName || !modelName
                    ? exports.getInputTypeName(typeInfo.type, dmmfDocument)
                    : typeInfo.type.replace(modelName, typeName);
        }
    }
    else {
        GraphQLType = typeInfo.type;
    }
    if (typeInfo.isList) {
        GraphQLType = `[${GraphQLType}]`;
    }
    return GraphQLType;
};
exports.mapScalarToType = (scalar) => {
    switch (scalar) {
        case 'DateTime': {
            return 'Date';
        }
        // TODO: use proper uuid graphql scalar
        case 'UUID': {
            return 'String';
        }
        case 'Boolean':
        case 'String': {
            return scalar;
        }
        case 'ID':
        case 'Int':
        case 'Float': {
            return `${scalar}`;
        }
        case 'Json': {
            return `GraphQLJSON`;
        }
        default: {
            throw new Error(`Unrecognized scalar type: ${scalar}`);
        }
    }
};
exports.camelCase = (str) => {
    return str[0].toLowerCase() + str.slice(1);
};
exports.pascalCase = (str) => {
    return str[0].toUpperCase() + str.slice(1);
};
const getInputKeywordPhrasePosition = (inputTypeName) => {
    const inputParseResult = [
        'Create',
        'OrderBy',
        'Update',
        'Upsert',
        'ScalarWhere',
        'Where',
        'Filter',
    ]
        .map((inputKeyword) => inputTypeName.search(inputKeyword))
        .filter((position) => position >= 0);
    if (inputParseResult.length === 0) {
        return;
    }
    const keywordPhrasePosition = inputParseResult[0];
    return keywordPhrasePosition;
};
exports.getModelNameFromInputType = (inputTypeName) => {
    const keywordPhrasePosition = getInputKeywordPhrasePosition(inputTypeName);
    if (!keywordPhrasePosition) {
        return;
    }
    const modelName = inputTypeName.slice(0, keywordPhrasePosition);
    return modelName;
};
exports.getInputTypeName = (originalInputName, dmmfDocument) => {
    const keywordPhrasePosition = getInputKeywordPhrasePosition(originalInputName);
    if (!keywordPhrasePosition) {
        return originalInputName;
    }
    const modelName = originalInputName.slice(0, keywordPhrasePosition);
    const typeNameRest = originalInputName.slice(keywordPhrasePosition);
    const modelTypeName = dmmfDocument.getModelTypeName(modelName);
    if (!modelTypeName) {
        return originalInputName;
    }
    return `${modelTypeName}${typeNameRest}`;
};
exports.cleanDocsString = (documentation) => {
    if (!documentation) {
        return;
    }
    let cleanedDocs = documentation;
    cleanedDocs = cleanedDocs.replace(helpers_1.modelAttributeRegex, '');
    cleanedDocs = cleanedDocs.replace(helpers_1.fieldAttributeRegex, '');
    cleanedDocs = cleanedDocs.split('"').join('\\"');
    cleanedDocs = cleanedDocs.split('\r').join('');
    cleanedDocs = cleanedDocs.split('\\r').join('');
    cleanedDocs = cleanedDocs.split('\n').join('');
    cleanedDocs = cleanedDocs.split('\\n').join('');
    return cleanedDocs;
};
exports.toUnixPath = (maybeWindowsPath) => {
    return maybeWindowsPath.split('\\').join('/');
};
//# sourceMappingURL=helpers.js.map