"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const helpers_1 = require("../helpers");
const args_class_1 = tslib_1.__importDefault(require("../args-class"));
const config_1 = require("../config");
const imports_1 = require("../imports");
const saveSourceFile_1 = tslib_1.__importDefault(require("../../utils/saveSourceFile"));
const generateRelationsResolverClassesFromModel = async (project, baseDirPath, model, mapping, outputType, dmmfDocument, options) => {
    const resolverName = `${model.typeName}RelationsResolver`;
    const rootArgName = helpers_1.camelCase(model.typeName);
    const relationFields = model.fields.filter((field) => field.relationName);
    const singleIdField = model.fields.find((field) => field.isId);
    const singleUniqueField = model.fields.find((field) => field.isUnique);
    const singleFilterField = singleIdField !== null && singleIdField !== void 0 ? singleIdField : singleUniqueField;
    const compositeIdFields = model.fields.filter((field) => model.idFields.includes(field.name));
    const compositeUniqueFields = model.fields.filter((field) => { var _a; return (_a = 
    // taking first unique group is enough to fetch entity
    model.uniqueFields[0]) === null || _a === void 0 ? void 0 : _a.includes(field.name); });
    const compositeFilterFields = compositeIdFields.length > 0 ? compositeIdFields : compositeUniqueFields;
    const resolverDirPath = path_1.default.resolve(baseDirPath, config_1.resolversFolderName, config_1.relationsResolversFolderName, model.typeName);
    const filePath = path_1.default.resolve(resolverDirPath, `${resolverName}.ts`);
    const sourceFile = project.createSourceFile(filePath, undefined, {
        overwrite: true,
    });
    const methodsInfo = await Promise.all(relationFields.map(async (field) => {
        const outputTypeField = outputType.fields.find((it) => it.name === field.name);
        let argsTypeName;
        if (outputTypeField.args.length > 0) {
            argsTypeName = await args_class_1.default(project, resolverDirPath, outputTypeField.args, `${model.typeName}${helpers_1.pascalCase(field.name)}Args`, dmmfDocument, options);
        }
        return { field, fieldDocs: field.docs, argsTypeName };
    }));
    const argTypeNames = methodsInfo
        .filter((it) => it.argsTypeName !== undefined)
        .map((it) => it.argsTypeName);
    const barrelExportSourceFile = project.createSourceFile(path_1.default.resolve(resolverDirPath, config_1.argsFolderName, 'index.ts'), undefined, { overwrite: true });
    if (argTypeNames.length) {
        imports_1.generateArgsBarrelFile(barrelExportSourceFile, argTypeNames);
        await saveSourceFile_1.default(barrelExportSourceFile);
    }
    imports_1.generateNestJSRelationsImport(sourceFile);
    imports_1.generateModelsImports(sourceFile, [...relationFields.map((field) => field.type), model.name].map((typeName) => dmmfDocument.isModelName(typeName)
        ? dmmfDocument.getModelTypeName(typeName)
        : typeName), 3);
    imports_1.generateClassTransformerImport(sourceFile);
    imports_1.generateArgsImports(sourceFile, argTypeNames, 0);
    sourceFile.addClass({
        name: resolverName,
        isExported: true,
        decorators: [
            {
                name: 'Resolver',
                arguments: [`() => ${model.typeName}`],
            },
        ],
        methods: methodsInfo.map(({ field, fieldDocs, argsTypeName }) => {
            var _a;
            let whereConditionString = '';
            // TODO: refactor to AST
            if (singleFilterField) {
                whereConditionString = `
            ${singleFilterField.name}: ${rootArgName}.${singleFilterField.name},
          `;
            }
            else if (compositeFilterFields.length > 0) {
                whereConditionString = `
            ${compositeFilterFields.map((it) => it.name).join('_')}: {
              ${compositeFilterFields
                    .map((idField) => `${idField.name}: ${rootArgName}.${idField.name},`)
                    .join('\n')}
            },
          `;
            }
            else {
                throw new Error(`Unexpected error happened on generating 'whereConditionString' for ${model.typeName} relation resolver`);
            }
            return {
                name: (_a = field.typeFieldAlias) !== null && _a !== void 0 ? _a : field.name,
                isAsync: true,
                returnType: `Promise<${field.fieldTSType}>`,
                decorators: [
                    {
                        name: 'ResolveField',
                        arguments: [
                            `() => ${field.typeGraphQLType}`,
                            `{
                  ${[
                                `nullable: ${!field.isRequired}`,
                                `description: ${fieldDocs ? `"${fieldDocs}"` : 'undefined'}`,
                                ...(!argsTypeName
                                    ? [
                                        `complexity: ({ childComplexity }) => 1 * childComplexity`,
                                    ]
                                    : [
                                        `complexity: ({ args, childComplexity }) => ((args.take + (args.skip ?? 0)) ?? 1) * childComplexity`,
                                    ]),
                            ].join(',\n')}
                }`,
                        ],
                    },
                ],
                parameters: [
                    {
                        name: rootArgName,
                        type: model.typeName,
                        decorators: [{ name: 'Root', arguments: [] }],
                    },
                    {
                        name: 'ctx',
                        // TODO: import custom `ContextType`
                        type: 'any',
                        decorators: [{ name: 'Context', arguments: [] }],
                    },
                    ...(!argsTypeName
                        ? []
                        : [
                            {
                                name: 'args',
                                type: argsTypeName,
                                decorators: [{ name: 'Args', arguments: [] }],
                            },
                        ]),
                ],
                // TODO: refactor to AST
                statements: [
                    `return plainToClass(${field.typeGraphQLType.replace(/\[|\]/g, '')}, await ctx.prisma.${helpers_1.camelCase(model.name)}.findOne({
              where: {${whereConditionString}},
            }).${field.name}(${argsTypeName ? 'args' : '{}'}) as ${field.typeGraphQLType});`,
                ],
            };
        }),
    });
    await saveSourceFile_1.default(sourceFile);
    return { modelName: model.typeName, resolverName, argTypeNames };
};
exports.default = generateRelationsResolverClassesFromModel;
//# sourceMappingURL=relations.js.map