"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateInputTypeClassFromType = exports.generateOutputTypeClassFromType = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const config_1 = require("./config");
const imports_1 = require("./imports");
const saveSourceFile_1 = tslib_1.__importDefault(require("../utils/saveSourceFile"));
exports.generateOutputTypeClassFromType = async (project, dirPath, type, dmmfDocument, options) => {
    const fileDirPath = path_1.default.resolve(dirPath, config_1.outputsFolderName);
    const filePath = path_1.default.resolve(fileDirPath, `${type.typeName}.ts`);
    const sourceFile = project.createSourceFile(filePath, undefined, {
        overwrite: true,
    });
    const fieldArgsTypeNames = type.fields
        .filter((it) => it.argsTypeName)
        .map((it) => it.argsTypeName);
    imports_1.generateNestJSModelImport(sourceFile);
    imports_1.generateGraphQLScalarImport(sourceFile);
    imports_1.generatePrismaJsonTypeImport(sourceFile, options, 2);
    imports_1.generateClassTransformerDTOImport(sourceFile);
    imports_1.generateArgsImports(sourceFile, fieldArgsTypeNames, 0);
    imports_1.generateOutputsImports(sourceFile, type.fields
        .filter((field) => field.outputType.kind === 'object')
        .map((field) => field.outputType.type), 1);
    sourceFile.addClass({
        name: type.typeName,
        isExported: true,
        decorators: [
            {
                name: 'ObjectType',
                arguments: [
                    `{
            isAbstract: true,
            description: undefined,
          }`,
                ],
            },
        ],
        properties: type.fields.map((field) => {
            const isRequired = field.outputType.isRequired;
            return {
                name: field.name,
                type: field.fieldTSType,
                hasExclamationToken: isRequired,
                hasQuestionToken: !isRequired,
                trailingTrivia: '\r\n',
                decorators: [
                    ...(field.outputType.kind === 'object'
                        ? [
                            {
                                name: 'ClassTransformer__Type',
                                arguments: [`() => ${field.outputType.type}`],
                            },
                        ]
                        : []),
                    {
                        name: 'Field',
                        arguments: [
                            `() => ${field.typeGraphQLType}`,
                            `{
                  nullable: ${!isRequired},
                  description: undefined
                }`,
                        ],
                    },
                ],
            };
        }),
    });
    await saveSourceFile_1.default(sourceFile);
};
exports.generateInputTypeClassFromType = async (project, dirPath, inputType, _dmmfDocument, options) => {
    const filePath = path_1.default.resolve(dirPath, config_1.inputsFolderName, `${inputType.typeName}.ts`);
    const sourceFile = project.createSourceFile(filePath, undefined, {
        overwrite: true,
    });
    imports_1.generateNestJSInputImport(sourceFile);
    imports_1.generateGraphQLScalarImport(sourceFile);
    imports_1.generatePrismaJsonTypeImport(sourceFile, options, 2);
    imports_1.generateClassTransformerDTOImport(sourceFile);
    imports_1.generateInputsImports(sourceFile, inputType.fields
        .filter((field) => field.selectedInputType.kind === 'object')
        .map((field) => field.selectedInputType.type)
        .filter((fieldType) => fieldType !== inputType.typeName));
    imports_1.generateEnumsImports(sourceFile, inputType.fields
        .map((field) => field.selectedInputType)
        .filter((fieldType) => fieldType.kind === 'enum')
        .map((fieldType) => fieldType.type), 2);
    const mappedFields = inputType.fields.filter((field) => field.hasMappedName);
    sourceFile.addClass({
        name: inputType.typeName,
        isExported: true,
        decorators: [
            {
                name: 'InputType',
                arguments: [
                    `{
            isAbstract: true,
            description: undefined,
          }`,
                ],
            },
        ],
        properties: inputType.fields.map((field) => {
            const isOptional = !field.selectedInputType.isRequired;
            return {
                name: field.name,
                type: field.fieldTSType,
                hasExclamationToken: !isOptional,
                hasQuestionToken: isOptional,
                trailingTrivia: '\r\n',
                decorators: field.hasMappedName
                    ? []
                    : [
                        ...(field.selectedInputType.kind === 'object'
                            ? [
                                {
                                    name: 'ClassTransformer__Type',
                                    arguments: [`() => ${field.selectedInputType.type}`],
                                },
                            ]
                            : []),
                        {
                            name: 'Field',
                            arguments: [
                                `() => ${field.typeGraphQLType}`,
                                `{
                      nullable: ${isOptional},
                      description: undefined
                    }`,
                            ],
                        },
                    ],
            };
        }),
        getAccessors: mappedFields.map((field) => {
            return {
                name: field.typeName,
                type: field.fieldTSType,
                hasExclamationToken: field.selectedInputType.isRequired,
                hasQuestionToken: !field.selectedInputType.isRequired,
                trailingTrivia: '\r\n',
                statements: [`return this.${field.name};`],
                decorators: [
                    ...(field.selectedInputType.kind === 'object'
                        ? [
                            {
                                name: 'ClassTransformer__Type',
                                arguments: [`() => ${field.selectedInputType.type}`],
                            },
                        ]
                        : []),
                    {
                        name: 'Field',
                        arguments: [
                            `() => ${field.typeGraphQLType}`,
                            `{
                  nullable: ${!field.selectedInputType.isRequired},
                  description: undefined
                }`,
                        ],
                    },
                ],
            };
        }),
        setAccessors: mappedFields.map((field) => {
            return {
                name: field.typeName,
                type: field.fieldTSType,
                hasExclamationToken: field.selectedInputType.isRequired,
                hasQuestionToken: !field.selectedInputType.isRequired,
                trailingTrivia: '\r\n',
                parameters: [{ name: field.name, type: field.fieldTSType }],
                statements: [`this.${field.name} = ${field.name};`],
            };
        }),
    });
    await saveSourceFile_1.default(sourceFile);
};
//# sourceMappingURL=type-class.js.map